package com.huimv.wine.utils;

import com.huimv.wine.config.KeyUrlConfig;
import org.apache.ibatis.annotations.Case;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.StandardCharsets;

@Component
public class KeyUtil {
    private static String serverPrivate;

    private static String serverPublic;

    private static String clientPrivate;

    private static String clientPublic;

    public static void setServerPrivate() {
//        KeyUrlConfig keyUrlConfig = SpringContextUtil.getBean(KeyUrlConfig.class);
//        String serverPrivateUrl = keyUrlConfig.getPair1PrivateUrl();
        InputStream inputStream = KeyUtil.class.getClassLoader().getResourceAsStream("certs/server-private.pem");
        if (inputStream != null) {
            try (BufferedReader reader = new BufferedReader(new InputStreamReader(inputStream, StandardCharsets.UTF_8))) {
                StringBuilder builder = new StringBuilder();
                String line;
                while ((line = reader.readLine()) != null) {
                    builder.append(line).append(System.lineSeparator());
                }
                serverPrivate = extractPrivateKeyFromString(builder.toString());
            } catch (IOException e) {
                e.printStackTrace();
                serverPrivate = null;
            }
        } else {
            serverPrivate = null;
        }
    }

    public static String extractPrivateKeyFromString(String publicKeyString){
        String startTag = "-----BEGIN RSA PRIVATE KEY-----";
        String endTag = "-----END RSA PRIVATE KEY-----";

        int start = publicKeyString.indexOf(startTag) + startTag.length();
        int end = publicKeyString.indexOf(endTag);

        if (start != -1 && end != -1) {
            return publicKeyString.substring(start, end).replaceAll("\\s", ""); // 去除空格和换行符
        } else {
            throw new IllegalArgumentException("Invalid public key string format.");
        }
    }

    public static String extractPublicKeyFromString(String publicKeyString) {
        String startTag = "-----BEGIN PUBLIC KEY-----";
        String endTag = "-----END PUBLIC KEY-----";

        int start = publicKeyString.indexOf(startTag) + startTag.length();
        int end = publicKeyString.indexOf(endTag);

        if (start != -1 && end != -1) {
            return publicKeyString.substring(start, end).replaceAll("\\s", ""); // 去除空格和换行符
        } else {
            throw new IllegalArgumentException("Invalid public key string format.");
        }

    }

    public static String getServerPrivate() {
        if (serverPrivate == null) {
            setServerPrivate();
        }
        return serverPrivate;
    }

    public static void setServerPublic() {
        KeyUrlConfig keyUrlConfig = SpringContextUtil.getBean(KeyUrlConfig.class);
        String serverPublicUrl = keyUrlConfig.getPair2PublicUrl();
        InputStream inputStream = KeyUtil.class.getClassLoader().getResourceAsStream(serverPublicUrl);
        if (inputStream != null) {
            try (BufferedReader reader = new BufferedReader(new InputStreamReader(inputStream, StandardCharsets.UTF_8))) {
                StringBuilder builder = new StringBuilder();
                String line;
                while ((line = reader.readLine()) != null) {
                    builder.append(line).append(System.lineSeparator());
                }
                serverPublic = extractPublicKeyFromString(builder.toString());
            } catch (IOException e) {
                e.printStackTrace();
                serverPublic = null;
            }
        } else {
            serverPublic = null;
        }
    }

    public static String getServerPublic() {
        if (serverPublic == null) {
            setServerPublic();
        }
        return serverPublic;
    }


    public static void setClientPrivate() {
        KeyUrlConfig keyUrlConfig = SpringContextUtil.getBean(KeyUrlConfig.class);
        String clientPrivateUrl = keyUrlConfig.getPair2PrivateUrl();
        InputStream inputStream = KeyUtil.class.getClassLoader().getResourceAsStream(clientPrivateUrl);
        if (inputStream != null) {
            try (BufferedReader reader = new BufferedReader(new InputStreamReader(inputStream, StandardCharsets.UTF_8))) {
                StringBuilder builder = new StringBuilder();
                String line;
                while ((line = reader.readLine()) != null) {
                    builder.append(line).append(System.lineSeparator());
                }
                clientPrivate = builder.toString();
            } catch (IOException e) {
                e.printStackTrace();
                clientPrivate = null;
            }
        } else {
            clientPrivate = null;
        }
    }

    public static String getClientPrivate() {
        if (clientPrivate == null) {
            setClientPrivate();
        }
        return clientPrivate;
    }


    public static void setClientPublic() {
//        KeyUrlConfig keyUrlConfig = SpringContextUtil.getBean(KeyUrlConfig.class);
//        String clientPublicUrl = keyUrlConfig.getPair1PublicUrl();
        InputStream inputStream = KeyUtil.class.getClassLoader().getResourceAsStream("certs/client-public.pem");
        if (inputStream != null) {
            try (BufferedReader reader = new BufferedReader(new InputStreamReader(inputStream, StandardCharsets.UTF_8))) {
                StringBuilder builder = new StringBuilder();
                String line;
                while ((line = reader.readLine()) != null) {
                    builder.append(line).append(System.lineSeparator());
                }
                clientPublic = builder.toString();
            } catch (IOException e) {
                e.printStackTrace();
                clientPublic = null;
            }
        } else {
            clientPublic = null;
        }
    }

    public static String getClientPublic() {
        if (clientPublic == null) {
            setClientPublic();
        }
        return clientPublic;
    }
}
