package com.huimv.wine.ws.su;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.wine.common.EventWsErrUtil;
import com.huimv.wine.entity.Manager;
import com.huimv.wine.entity.Operation;
import com.huimv.wine.entity.vo.WsEvent;
import com.huimv.wine.entity.vo.workQueryResVo;
import com.huimv.wine.mapper.ManagerMapper;
import com.huimv.wine.mapper.OperationMapper;
import com.huimv.wine.mapper.WorkerMapper;
import com.huimv.wine.utils.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.websocket.Session;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Component
public class SuperRole {

    @Autowired
    private ManagerMapper managerMapper;
    @Autowired
    private OperationMapper operationMapper;
    @Autowired
    private WorkerMapper workerMapper;
    public void handle(String event, JSONObject data, Session session, Manager manager) {
        switch (event) {
            case "superRoleAdminQuery":
                query(session, manager, data);
                break;
            case "superRoleAdminUpdate":
                Update(session, manager, data);
                break;
            case "superRoleAdminDelete":
                JSONArray array = data.getJSONArray("data");
                List<String> list = new ArrayList<>();
                for (int i = 0; i < array.size(); i++) {
                    list.add(array.getString(i));
                }
                Delete(session,manager,list);
                break;
            case "superRoleAdminAdd":
                Add(session, manager, data);
                break;
            case "superRoleWorkerQuery":
                Query(session, manager, data);
                break;
            default:
                WsEvent wsEvent = EventWsErrUtil.getWsErr("unrecognized event");
                WebsocketManagerUtil.sendMessage(session, wsEvent);
                break;
        }
    }

    public void query(Session session,Manager manager,JSONObject data) {
        if (!manager.getIsSuper()) {
            WsEvent wsEvent = new WsEvent("superRoleAdminQueryRes", Result.fail("权限不足"));
            WebsocketManagerUtil.sendMessage(session, wsEvent);
            return;
        }
        if (StringUtils.isBlank(data.getString("cond")) || data.getInteger("page") == null || data.getInteger("limit") == null) {
            WsEvent wsEvent = new WsEvent("superRoleAdminQueryRes", Result.fail("参数错误"));
            WebsocketManagerUtil.sendMessage(session, wsEvent);
            return;
        }
        QueryWrapper<Manager> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("super", 0).and(i -> i.like("name", data.getString("cond")).or()
                .like("account", data.getString("cond")));
        Page<Manager> page = new Page(data.getInteger("page"), data.getInteger("limit"));
        Page<Manager> managerPage = managerMapper.selectPage(page, queryWrapper);
        WsEvent wsEvent = new WsEvent("superRoleAdminQueryRes", Result.success(managerPage));
        WebsocketManagerUtil.sendMessage(session, wsEvent);

    }
    public void Update(Session session,Manager manager,JSONObject data) {
        if (!manager.getIsSuper()) {
            WsEvent wsEvent = new WsEvent("superRoleAdminUpdateRes", Result.fail("权限不足"));
            WebsocketManagerUtil.sendMessage(session, wsEvent);
            return;
        }
        if (StringUtils.isBlank(data.getString("id")) || StringUtils.isBlank(data.getString("name"))
                || StringUtils.isBlank(data.getString("account"))||StringUtils.isBlank(data.getString("password"))) {
            WsEvent wsEvent = new WsEvent("superRoleAdminUpdateRes", Result.fail("参数错误"));
            WebsocketManagerUtil.sendMessage(session, wsEvent);
            return;
        }
        String id = data.getString("id");
        Manager manager1 = managerMapper.selectById(id);
        if (ObjectUtil.isEmpty(manager1)) {
            WsEvent wsEvent = new WsEvent("superRoleAdminUpdateRes", Result.fail("管理员不存在"));
            WebsocketManagerUtil.sendMessage(session, wsEvent);
            return;
        }
        String serverPrivate = KeyUtil.getServerPrivate();
        String realPassword = RSAUtil.decrypt1(data.getString("password"), serverPrivate);
        if (StringUtils.isBlank(realPassword)) {
            WsEvent wsEvent = new WsEvent("superRoleAdminUpdateRes", Result.fail("解密失败"));
            WebsocketManagerUtil.sendMessage(session, wsEvent);
            return;
        }
        manager1.setAccount(data.getString("account"));
        manager1.setName(data.getString("name"));
        int update = managerMapper.updateById(manager1);
        if (update == 0) {
            WsEvent wsEvent = new WsEvent("superRoleAdminUpdateRes", Result.fail("更新失败"));
            WebsocketManagerUtil.sendMessage(session, wsEvent);
            return;
        }
        WsEvent wsEvent = new WsEvent("superRoleAdminUpdateRes", Result.success(null));
        WebsocketManagerUtil.sendMessage(session, wsEvent);
        Operation operation = new Operation();
        operation.setUserType(0);
        operation.setUserId(manager.getId());
        operation.setOpType(1);
        operation.setLevel(0);
        operation.setTime(new Date());
        operation.setDescribe("更新管理员信息");
        operationMapper.insert(operation);
    }

    public void Delete(Session session, Manager manager, List<String> data) {
        if (!manager.getIsSuper()) {
            WsEvent wsEvent = new WsEvent("superRoleAdminDeleteRes", Result.fail("权限不足"));
            WebsocketManagerUtil.sendMessage(session, wsEvent);
            return;
        }
         managerMapper.deleteBatchIds(data);
        WsEvent wsEvent = new WsEvent("superRoleAdminDeleteRes", Result.success(null));
        WebsocketManagerUtil.sendMessage(session, wsEvent);
        Operation operation = new Operation();
        operation.setUserType(0);
        operation.setUserId(manager.getId());
        operation.setOpType(2);
        operation.setLevel(1);
        operation.setTime(new Date());
        operation.setDescribe("删除管理员");
        operationMapper.insert(operation);
    }

    public void Add(Session session,Manager manager,JSONObject data) {
        if (!manager.getIsSuper()) {
            WsEvent wsEvent = new WsEvent("superRoleAdminAddRes", Result.fail("权限不足"));
            WebsocketManagerUtil.sendMessage(session, wsEvent);
            return;
        }
        if (StringUtils.isBlank(data.getString("name")) || data.getInteger("account") == null || data.getInteger("password") == null) {
            WsEvent wsEvent = new WsEvent("superRoleAdminAddRes", Result.fail("数据不可为空"));
            WebsocketManagerUtil.sendMessage(session, wsEvent);
            return;
        }
        String serverPrivate = KeyUtil.getServerPrivate();
        String realPassword = RSAUtil.decrypt1(data.getString("password"), serverPrivate);
        if (StringUtils.isBlank(realPassword)) {
            WsEvent wsEvent = new WsEvent("superRoleAdminAddRes", Result.fail("解密失败"));
            WebsocketManagerUtil.sendMessage(session, wsEvent);
            return;
        }
        String id = Utils.randomString(16);
        Manager manager1 = new Manager();
        manager1.setId(id);
        manager1.setName(data.getString("name"));
        manager1.setAccount(data.getString("account"));
        manager1.setPassword(realPassword);
        int insert = managerMapper.insert(manager1);
        if (insert == 0) {
            WsEvent wsEvent = new WsEvent("superRoleAdminAddRes", Result.fail("添加失败"));
            WebsocketManagerUtil.sendMessage(session, wsEvent);
            return;
        }
        WsEvent wsEvent = new WsEvent("superRoleAdminAddRes", Result.success(null));
        WebsocketManagerUtil.sendMessage(session, wsEvent);
        Operation operation = new Operation();
        operation.setUserType(0);
        operation.setUserId(manager.getId());
        operation.setOpType(0);
        operation.setLevel(0);
        operation.setTime(new Date());
        operation.setDescribe("新增管理员");
        operationMapper.insert(operation);
    }

    public void Query(Session session,Manager manager,JSONObject data) {
        if (!manager.getIsSuper()) {
            WsEvent wsEvent = new WsEvent("superRoleWorkerQueryRes", Result.fail("权限不足"));
            WebsocketManagerUtil.sendMessage(session, wsEvent);
            return;
        }
        String manger = data.getString("manger");
        String cond = data.getString("cond");
        Integer page = data.getInteger("page");
        Integer limit = data.getInteger("limit");
        if ("".equals(manger)) {
            Page<workQueryResVo> voPage = new Page(page, limit);
            Page<workQueryResVo> workerQuery = workerMapper.WorkerQuery(voPage, cond);
            WsEvent wsEvent = new WsEvent("superRoleWorkerQueryRes", Result.success(workerQuery));
            WebsocketManagerUtil.sendMessage(session, wsEvent);
        } else {
            Page<workQueryResVo> voPage = new Page(page, limit);
            Page<workQueryResVo> workerQuery = workerMapper.WorkerQuery2(voPage, manger, cond);
            WsEvent wsEvent = new WsEvent("superRoleWorkerQueryRes", Result.success(workerQuery));
            WebsocketManagerUtil.sendMessage(session, wsEvent);
        }
    }
}
